/*
  Name Tag
    あかつきみさき

  このスクリプトについて
    任意のテキストをリストに登録し,リスト選択でレイヤーに名前・コメントを追加するスクリプト.

  使用方法
    1.ファイル→スクリプト→スクリプトファイルの実行から実行.
    2.ScriptUIフォルダに入れてから,ウィンドウより実行.

  動作保障環境
    Adobe After Effects CS6以上

  バージョン情報
    2021/12/23 Ver 1.0.0 Release
*/

(function() {
    Settings.prototype.initSetting = function(sectionName, keyName, value) {
        if (!app.settings.haveSetting(sectionName, keyName)) {
            app.settings.saveSetting(sectionName, keyName, value);
        }
        return app.settings.getSetting(sectionName, keyName);
    };

    var NAME_LIST = app.settings.initSetting("Name Tag", "NAME_LIST", "");

    function buildUI(thisObj) {
        var layerbmWin;
        if (thisObj instanceof Panel) {
            layerbmWin = thisObj;
        } else {
            layerbmWin = new Window("palette", "Name Tag", undefined, {
                resizeable: true
            });
        }

        layerbmWin.orientation = "column";
        layerbmWin.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.TOP];
        layerbmWin.margins = [0, 0, 0, 0];
        layerbmWin.spacing = 1;
        layerbmWin.size = [400, 500];

        var btnGrp = layerbmWin.add("group");
        btnGrp.margins = [0, 0, 0, 0];
        btnGrp.spacing = 0;
        btnGrp.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.TOP];
        btnGrp.orientation = "row";

        var addBtn = btnGrp.add("button", undefined, "Add");
        addBtn.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        addBtn.helpTip = "";

        var clearBtn = btnGrp.add("button", undefined, "Clear");
        clearBtn.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        clearBtn.helpTip = "";

        var helpBtn = btnGrp.add("button", undefined, "?");
        helpBtn.alignment = [ScriptUI.Alignment.RIGHT, ScriptUI.Alignment.FILL];
        helpBtn.minimumSize.width = helpBtn.maximumSize.width = 25;
        helpBtn.helpTip = "";

        var listGrp = layerbmWin.add("group");
        listGrp.margins = [0, 0, 0, 0];
        listGrp.spacing = 0;
        listGrp.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        listGrp.orientation = "column";

        var proLb = listGrp.add("listbox");
        proLb.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];

        var footerGrp = layerbmWin.add("group");
        footerGrp.margins = [0, 0, 0, 0];
        footerGrp.spacing = 0;
        footerGrp.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.BOTTOM];
        footerGrp.orientation = "row";

        var nameBtn = footerGrp.add("iconbutton", undefined, undefined, {
            toggle: true
        });
        nameBtn.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        nameBtn.text = "Name";
        nameBtn.helpTip = "";
        nameBtn.value = true;

        var commentBtn = footerGrp.add("iconbutton", undefined, undefined, {
            toggle: true
        });
        commentBtn.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        commentBtn.text = "Comment";
        commentBtn.helpTip = "";


        var curList = NAME_LIST.split(",");
        for (var i = 0; i < curList.length; i++) {
            if (curList[i] == "") {
                continue;
            }

            var tempItem = proLb.add("item", decodeURIComponent(curList[i]));
            tempItem.checked = true;
        }

        addBtn.onClick = function() {
            var testText = prompt("Text", "");
            if (testText == "" || testText == null) {
                return 0;
            }

            var tempItem = proLb.add("item", testText);
            tempItem.checked = true;

            var allList = [];
            for (var i = 0; i < proLb.items.length; i++) {
                allList[allList.length] = encodeURIComponent(proLb.items[i].text);
            }

            var saveText = allList.join(",");
            app.settings.saveSetting("Name Tag", "NAME_LIST", saveText);
        }

        clearBtn.onClick = function() {
            for (var i = proLb.items.length - 1; i >= 0; i--) {
                if (proLb.items[i].checked) {
                    continue;
                }

                proLb.remove(proLb.items[i]);
            }

            var allList = [];
            for (var i = 0; i < proLb.items.length; i++) {
                allList[allList.length] = encodeURIComponent(proLb.items[i].text);
            }

            var saveText = allList.join(",");
            app.settings.saveSetting("Name Tag", "NAME_LIST", saveText);
        }

        proLb.onChange = function() {
            if (this.selection == null) {
                return 0;
            }

            if (ScriptUI.environment.keyboardState.ctrlKey) {
                this.selection.checked = !this.selection.checked;
                this.selection = null;
                return 0;
            }

            if (ScriptUI.environment.keyboardState.altKey) {
                proLb.remove(this.selection);
                this.selection = null;
                return 0;
            }

            var actComp = app.project.activeItem;
            if (!(actComp instanceof CompItem)) {
                return 0;
            }

            if (!nameBtn.value && !commentBtn.value) {
                alert("挿入先が選択されていません.\n下部の「Name」か「Comment」の選択をしてください.");
                return 0;
            }

            var selLayers = actComp.selectedLayers;
            for (var j = 0; j < selLayers.length; j++) {
                if (nameBtn.value) {
                    selLayers[j].name = this.selection.text;
                }

                if (commentBtn.value) {
                    selLayers[j].comment = this.selection.text;
                }
            }

            this.selection = null;
        }

        helpBtn.onClick = function() {
            alert("Name Tag 1.0.0\nあかつきみさき\n\n任意のテキストをリストに登録し,リスト選択でレイヤーに名前・コメントを追加するスクリプト.\n\nクリック: 該当レイヤーの選択\nCtrl+クリック: 該当項目の保護(チェックマーク)\nAlt+クリック: 該当項目の削除\nAdd: 選択レイヤーをリストに反映\nClear: 保護されていないリスト項目をクリア", "Name Tag");
        }

        layerbmWin.layout.layout();
        layerbmWin.onResizing = layerbmWin.onResize = function() {
            this.layout.resize();
        }

        if (!(thisObj instanceof Panel)) {
            layerbmWin.center();
            layerbmWin.show();
        }

        return 0;
    }

    buildUI(this);

    return 0;
}).call(this);